﻿using C1.WPF.Grid;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace CheckBoxColumnNET6
{
    /// <summary>
    /// Interaction logic for ConditionalFormatting.xaml
    /// </summary>
    public partial class ConditionalFormatting : UserControl
    {
        public ConditionalFormatting()
        {
            InitializeComponent();
            var data = Customer.GetCustomerList(100);
            grid.ItemsSource = data;
            grid.Columns[5].DataMap = new GridDataMap() { ItemsSource = Customer.GetCountries(), DisplayMemberPath = "Value", SelectedValuePath = "Key" };
            grid.CellFactory = new MyCellFactory();
            grid.MinColumnWidth = 85;
        }
    }

    public class MyCellFactory : GridCellFactory
    {
        public override void PrepareCell(GridCellType cellType, GridCellRange range, GridCellView cell, Thickness internalBorders)
        {
            base.PrepareCell(cellType, range, cell, internalBorders);
            var orderCountColumn = Grid.Columns["OrderCount"];
            if (cellType == GridCellType.Cell && range.Column == orderCountColumn.Index)
            {
                var cellValue = Grid[range.Row, range.Column] as int?;
                if (cellValue.HasValue)
                {
                    cell.Background = new SolidColorBrush(cellValue < 50.0 ? Color.FromRgb(0xFF, 0x70, 0x70) : Color.FromRgb(0x8E, 0xE9, 0x8E));
                }
            }

            try
            {
                if (cellType == GridCellType.Cell && Grid.Columns[range.Column].Header == "Test")
                {
                    if (range.Row % 2 == 0)
                    {
                        cell.Content = CreateCheckBox("Test" + range.Row, false, false);
                    }
                    else
                    {
                        cell.Content = CreateCheckBox("Test" + range.Row, true, false);
                    }
                }
                if(cellType==GridCellType.Cell && Grid.Columns[range.Column].Header== "OrderCount")
                {
                    var model = (Customer)(Grid.Rows[range.Row].DataItem);
                    if (range.Row % 2 == 0)
                    {
                        cell.Content = CreateCheckBox(model.OrderCount.ToString(), false, false);
                    }
                    else
                    {
                        cell.Content = CreateCheckBox(model.OrderCount.ToString(), true, false);
                    }
                }
            }
            catch
            {
                throw;
            }
        }

        /// <summary>
        /// Binds the content of the cell.
        /// </summary>
        /// <param name="cellType">Type of the cell.</param>
        /// <param name="range">The range.</param>
        /// <param name="cellContent">Content of the cell.</param>
        public override void BindCellContent(GridCellType cellType, GridCellRange range, FrameworkElement cellContent)
        {
            base.BindCellContent(cellType, range, cellContent);
            var orderTotalColumn = Grid.Columns["OrderTotal"];
            var orderCountColumn = Grid.Columns["OrderCount"];
            if (cellType == GridCellType.Cell && range.Column == orderTotalColumn.Index)
            {
                var label = cellContent as TextBlock;
                if (label != null)
                {
                    var cellValue = Grid[range.Row, range.Column] as double?;
                    if (cellValue.HasValue)
                    {
                        label.Foreground = new SolidColorBrush(cellValue < 5000.0 ? Colors.Red : Colors.Green);
                    }
                }
            }
            if (cellType == GridCellType.Cell && range.Column == orderCountColumn.Index)
            {
                var label = cellContent as CheckBox;
                if (label != null)
                {
                    var cellValue = Grid[range.Row, range.Column] as int?;
                    if (cellValue.HasValue)
                    {
                        label.Foreground = new SolidColorBrush(Colors.Black);
                    }
                }
            }


        }

        public override void UnbindCellContent(GridCellType cellType, GridCellRange range, FrameworkElement cellContent)
        {
            base.UnbindCellContent(cellType, range, cellContent);
            var label = cellContent as TextBlock;
            if (label != null)
            {
                label.ClearValue(Label.ForegroundProperty);
            }
        }

        private CheckBox CreateCheckBox(string Content, bool IsChecked, bool IsEnabled)
        {
            try
            {
                CheckBox chb = new CheckBox();
                chb.Content = Content;
                chb.IsChecked = IsChecked;
                chb.IsEnabled = IsEnabled;
                return chb;
            }
            catch (Exception)
            {

                throw;
            }
        }
        public override object GetCellContentType(GridCellType cellType, GridCellRange range)
        {
            double val;
            if (cellType == GridCellType.Cell && Grid.Columns[range.Column].Header == "Test")
            {
                return typeof(CheckBox);
            }
            if (cellType == GridCellType.Cell && range.Column == 4)
            {
                return typeof(CheckBox);
            }
            return base.GetCellContentType(cellType, range);
        }



    }
}
